@@ -49,8 +49,6 @@
 #include <exempi/xmp.h>
 #endif
 
-#define APPLICATION_SERVICE_NAME "org.gnome.eog.ApplicationService"
-
 static void eog_application_load_accelerators (void);
 static void eog_application_save_accelerators (void);
 
@@ -473,7 +471,7 @@ eog_application_get_instance (void)
 
 	if (!instance) {
 		instance = EOG_APPLICATION (g_object_new (EOG_TYPE_APPLICATION,
-							  "application-id", APPLICATION_SERVICE_NAME,
+							  "application-id", APPLICATION_ID,
 							  "flags", G_APPLICATION_HANDLES_OPEN,
 							  NULL));
 	}
@@ -531,6 +529,10 @@ eog_application_open_window (EogApplication  *application,
 	new_window = GTK_WIDGET (eog_application_get_empty_window (application));
 
 	if (new_window == NULL) {
+		/* Filter out fullscreen flags to avoid going fullscreen
+		 * with functions to leave fullscreen possibly being
+		 * disabled due to the empty model */
+		flags &= ~(EOG_STARTUP_FULLSCREEN | EOG_STARTUP_SLIDE_SHOW);
 		new_window = eog_window_new (flags);
 	}
 
