@@ -463,6 +463,9 @@ file_monitor_changed_cb (GFileMonitor *monitor,
 	case G_FILE_MONITOR_EVENT_PRE_UNMOUNT:
 	case G_FILE_MONITOR_EVENT_UNMOUNTED:
 	case G_FILE_MONITOR_EVENT_MOVED:
+	case G_FILE_MONITOR_EVENT_RENAMED:
+	case G_FILE_MONITOR_EVENT_MOVED_IN:
+	case G_FILE_MONITOR_EVENT_MOVED_OUT:
 		break;
 	}
 }
